<?php

namespace Teknisa\PLA\Modules\Medicao\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;


class Pla01000EfetivosService
{
    protected $crudService;
    protected $entityManager;
    protected $environment;
    protected $dataSourceManager;
    protected $connection;

    public function __construct($crudService, EntityManager $entityManager, $environment, DataSource\Manager $dataSourceManager) {
        $this->crudService       = $crudService;
        $this->entityManager     = $entityManager;
        $this->environment       = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection        = $entityManager->getConnection();
    }

    private function save($dataSet, $dataSourceName) {
        $dataSet = new DataSet($dataSourceName, $dataSet);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
        $dataSet = new DataSet($dataSourceName, $rowsToSave);
        $this->dataSourceManager->persist($dataSet);
    }

    public function getCardadia($cdfilial, $nrcardapio) {
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator"  => "=",
                "value" => $cdfilial
            ),
            array(
                "columnName" => "NRCARDAPIO",
                "operator"  => "=",
                "value" => $nrcardapio
            )
        );
        $rows = $this->crudService->getFilteredDataSet("cardadia", $conditions)->getRows();
        if (empty($rows)) {
            throw new \Exception("cardadiaNotFound", 1);
        }
        return $rows[0];
    }

    public function getRetirada($cdfilial, $nrcardapio) {
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator"  => "=",
                "value" => $cdfilial
            ),
            array(
                "columnName" => "NRCARDAPIO",
                "operator"  => "=",
                "value" => $nrcardapio
            )
        );
        return $this->crudService->getFilteredDataSet("retirada", $conditions)->getRows();
    }

    public function getTurnoclie($turno) {
       $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator"  => "=",
                "value" => $turno["CDFILIAL"]
            ),
            array(
                "columnName" => "CDSERVFILI",
                "operator"  => "=",
                "value" => $turno["CDSERVFILI"]
            ),
            array(
                "columnName" => "CDCLIENTE",
                "operator"  => "=",
                "value" => $turno["CDCLIENTE"]
            ),
            array(
                "columnName" => "DTFATTURNO",
                "operator"  => "=",
                "value" => $turno["DTFATTURNO"]
            ),
            array(
                "columnName" => "CDTURNO",
                "operator"  => "=",
                "value" => $turno["CDTURNO"]
            )
        );
        $rows = $this->crudService->getFilteredDataSet("turnoclie", $conditions)->getRows();
        if (!empty($rows)) {
            return $rows[0];
        }
        else {
            return array();
        }
    }

    public function getEfetclie($cliente) {
       $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator"  => "=",
                "value" => $cliente["CDFILIAL"]
            ),
            array(
                "columnName" => "CDSERVFILI",
                "operator"  => "=",
                "value" => $cliente["CDSERVFILI"]
            ),
            array(
                "columnName" => "CDCLIENTE",
                "operator"  => "=",
                "value" => $cliente["CDCLIENTE"]
            ),
            array(
                "columnName" => "DTEFETCLIE",
                "operator"  => "=",
                "value" => $cliente["DTEFETCLIE"]
            ),
            array(
                "columnName" => "CDSERVPLAN",
                "operator"  => "=",
                "value" => $cliente["CDSERVPLAN"]
            )
        );
        $rows = $this->crudService->getFilteredDataSet("efetclie", $conditions)->getRows();
        if (!empty($rows)) {
            return $rows[0];
        }
        else {
            return array();
        }
    }

    public function saveCardadia($servplan) {
        try {
            $cardadia = $this->getCardadia($servplan["CDFILIAL"], $servplan["NRCARDAPIO"]);

            $cardadia["QTCLIEFETCAR"] = $servplan["QTCLIEFETCAR"];
            $cardadia["QTFUNEFETCAR"] = $servplan["QTFUNEFETCAR"];
            $cardadia["QTDOAEFETCAR"] = $servplan["QTDOAEFETCAR"];
            $cardadia["QTCOMEFETCAR"] = $servplan["QTCOMEFETCAR"];

            $this->save(array($cardadia), "cardadia");
        } catch(\Exception $e) {
            throw $e;
        };
    }

    public function saveEfetclie($cliente) {
        $efetclie = $this->getEfetclie($cliente);
        if (!empty($efetclie)) {
            $this->updateEfetclie($efetclie, $cliente);
        } else {
            $this->insertEfetclie($cliente);
        }
        if($cliente['EFETIVOSRATEIO'] != null){
                $this->deleteItratefetclie($cliente);
                if(isset($cliente['rateioDataSource'])){
                    foreach($cliente['rateioDataSource'] as $itemRateio){
                        if($itemRateio['QTEFETIVOS'] != null || $itemRateio['QTEFETIVOS'] != ""){
                            $this->insertItratefetclie($itemRateio, $cliente);
                        }
                    }
                }
        }else{
            $this->deleteItratefetclie($cliente);
        }
    }

    public function updateEfetclie($efetclie, $cliente) {
        $efetclie["QTCOMEFETCLI"]    = $cliente["QTCOMEFETCLI"];
        $efetclie["VRSERVFATUEFCLI"] = $cliente["VRSERVFATUEFCLI"];
        $efetclie["QTVENDAEFETCLI"]  = $cliente["QTVENDAEFETCLI"];
        $efetclie["__is_new"]        = false;

        $this->save(array($efetclie), "efetclie");
    }

    public function insertEfetclie($cliente) {
        $efetclie = array();

        $efetclie["CDFILIAL"]        = $cliente["CDFILIAL"];
        $efetclie["CDSERVFILI"]      = $cliente["CDSERVFILI"];
        $efetclie["CDCLIENTE"]       = $cliente["CDCLIENTE"];
        $efetclie["DTEFETCLIE"]      = $cliente["DTEFETCLIE"];
        $efetclie["CDSERVPLAN"]      = $cliente["CDSERVPLAN"];
        $efetclie["__is_new"]        = true;

        $efetclie["QTCOMEFETCLI"]    = $cliente["QTCOMEFETCLI"];
        $efetclie["VRSERVFATUEFCLI"] = $cliente["VRSERVFATUEFCLI"];
        $efetclie["QTVENDAEFETCLI"]  = $cliente["QTVENDAEFETCLI"];

        $this->save(array($efetclie), "efetclie");
    }

    public function insertItratefetclie($itemRateio, $cliente) {
        $itratefetclie = array();
        $itratefetclie["CDFILIAL"]        = $cliente["CDFILIAL"];
        $itratefetclie["CDSERVFILI"]      = $cliente["CDSERVFILI"];
        $itratefetclie["CDCLIENTE"]       = $cliente["CDCLIENTE"];
        $itratefetclie["DTEFETCLIE"]      = $cliente["DTEFETCLIE"];
        $itratefetclie["CDSERVPLAN"]      = $cliente["CDSERVPLAN"];
        
        $itratefetclie["__is_new"]        = true;

        $itratefetclie['__is_new_code']       = true;
        $itratefetclie['__new_code_pk']       = $itemRateio['__new_code_pk'];
        $itratefetclie['__table_pk']          = $itemRateio['__table_pk'];
        $itratefetclie['__new_code_length']   = $itemRateio['__new_code_length'];
        $itratefetclie["NRITRATEFETCLIE"]     = $itemRateio["NRITRATEFETCLIE"];
        $itratefetclie["NRITEMRATEIOEFET"]    = $itemRateio["NRITEMRATEIOEFET"];
        $itratefetclie["QTEFETIVOS"]          = $itemRateio["QTEFETIVOS"];

        $this->save(array($itratefetclie), "itratefetclie");
    }
    
    public function saveTurnoclie($turno) {
        $turnoclie = $this->getTurnoclie($turno);
        if (!empty($turnoclie)) {
            if ($turno["QTCOMTURNCLI"] != 0) {
                $this->updateTurnoclie($turnoclie, $turno);
            } else {
                $this->deleteTurnoclie($turnoclie);
            }
        } else {
            if ($turno["QTCOMTURNCLI"] != 0) {
                $this->insertTurnoclie($turno);
            }
        }
    }

    public function updateTurnoclie($turnoclie, $turno) {
        $turnoclie["QTCOMTURNCLI"]    = $turno["QTCOMTURNCLI"];
        $turnoclie["__is_new"]        = false;

        $this->save(array($turnoclie), "turnoclie");
    }

    public function insertTurnoclie($turno) {
        $turnoclie = array();

        $turnoclie["CDFILIAL"]        = $turno["CDFILIAL"];
        $turnoclie["CDSERVFILI"]      = $turno["CDSERVFILI"];
        $turnoclie["CDTURNO"]         = $turno["CDTURNO"];
        $turnoclie["CDCLIENTE"]       = $turno["CDCLIENTE"];
        $turnoclie["DTFATTURNO"]      = $turno["DTFATTURNO"];
        $turnoclie["__is_new"]        = true;

        $turnoclie["QTCOMTURNCLI"]    = $turno["QTCOMTURNCLI"];
        $this->save(array($turnoclie), "turnoclie");
    }

    public function deleteTurnoclie($turnoclie) {
        $sql = "DELETE FROM 
                    TURNOCLIE 
                WHERE
                    CDFILIAL   = :CDFILIAL   AND
                    CDSERVFILI = :CDSERVFILI AND
                    CDTURNO    = :CDTURNO    AND
                    CDCLIENTE  = :CDCLIENTE  AND
                    DTFATTURNO = :DTFATTURNO
                ";
        $params = array(
            "CDFILIAL"   => $turnoclie["CDFILIAL"],
            "CDSERVFILI" => $turnoclie["CDSERVFILI"],
            "CDTURNO"    => $turnoclie["CDTURNO"],
            "CDCLIENTE"  => $turnoclie["CDCLIENTE"],
            "DTFATTURNO" => $turnoclie["DTFATTURNO"],
        );
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function deleteItratefetclie($cliente) {
        $sql = "DELETE FROM 
                    ITRATEFETCLIE 
                WHERE
                    CDFILIAL   = :CDFILIAL   AND
                    CDSERVFILI = :CDSERVFILI AND
                    DTEFETCLIE = :DTEFETCLIE AND
                    CDCLIENTE  = :CDCLIENTE  AND 
                    CDSERVPLAN = :CDSERVPLAN
                ";
        $params = array(
            "CDFILIAL"   => $cliente["CDFILIAL"],
            "CDSERVFILI" => $cliente["CDSERVFILI"],
            "DTEFETCLIE" => $cliente["DTEFETCLIE"],
            "CDCLIENTE"  => $cliente["CDCLIENTE"],
            "CDSERVPLAN" => $cliente["CDSERVPLAN"]
        );
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

}